import javax.sound.sampled.*;
import java.io.File;

public class AudioData
{
	private int[]   leftChannel  = null; // Sample-Daten des linken Kanals
    private int[]   rightChannel = null; // Sample-Daten des rechten Kanals
    private int     numChannels  = 0;    // Anzahl der Kanle
    private long    millis = 0;          // Dauer in Millisekunden
    private int     sampleDepth = 0;     // Bits pro Sample
    private float   sampleRate = 0;      // Samples pro Sekunde
    private boolean valid = false;       // true, wenn Audiodaten gltig
   
    public AudioData (File audiofile) 
    {
    	if (audiofile != null && audiofile.isFile()) try
   		{
 			AudioInputStream inputstream = AudioSystem.getAudioInputStream(audiofile);
			AudioFormat format = inputstream.getFormat();
			numChannels = format.getChannels();
			millis = (long)(1000 * inputstream.getFrameLength()/format.getFrameRate());
			sampleDepth = format.getSampleSizeInBits();
			sampleRate  = format.getSampleRate();
			
			int frameLength = (int)inputstream.getFrameLength();				
			
			leftChannel = new int[frameLength];
			if (numChannels == 2)
				rightChannel = new int[frameLength];
				
			byte frame[] = new byte[format.getFrameSize()];

			if (sampleDepth == 16)
			{					
				for (int i=0; i < frameLength; i++)
				{
					inputstream.read(frame, 0, frame.length);
					leftChannel[i] = toInt(frame[0], frame[1], format);
					if (numChannels == 2)
						rightChannel[i] = toInt(frame[2], frame[3], format);						
				}
			}
			else if (sampleDepth == 8)
			{
				for (int i=0; i < frameLength; i++)
				{
					inputstream.read(frame, 0, frame.length);
					leftChannel[i] = toInt(frame[0], format);
					if (numChannels == 2)
						rightChannel[i] = toInt(frame[1], format);						
				}
			}
			inputstream.close();
			valid = true;  // wenn wir hier angekommen sind, wurden gltige Audiodaten gelesen
		}
   		catch (Exception e)
   		{
   			System.out.println("Fehler beim ffnen von '" + audiofile + "'");
      		e.printStackTrace();
   		}
    }
    
/*    public void print ()
    {
    	for (int i=0; i < leftChannel.length; i++)
    	{
    		System.out.print(i + ": " + leftChannel[i]);
    		if (rightChannel != null)
	    		System.out.println(", " + rightChannel[i]);
    	}
    }*/
       
   
    public int     getNumChannels ()  {return numChannels;}   
    public long    getMillis ()       {return millis;}  
    public int[]   getLeftChannel ()  {return leftChannel;}
    public int[]   getRightChannel () {return rightChannel;}
    public void setLeftChannel(int[] lc) {leftChannel = lc;}
    public void setRightChannel(int[] rc) {rightChannel = rc;}  
    public boolean isValid ()         {return valid;}
    public float   getSampleRate ()   {return sampleRate;}
    public int     getSampleDepth ()  {return sampleDepth;}
  
    
	// wandelt ein einzelnes Byte gem angegebenem AudioFormat in eine Integer-Zahl um
	protected static int toInt (byte b, AudioFormat format)
	{
		int res;
		if (format.getEncoding() == AudioFormat.Encoding.PCM_UNSIGNED)
			res = b & 0xff;
		else
			res = b;
		if (format.getEncoding() == AudioFormat.Encoding.PCM_UNSIGNED)
			res -= 0x7f;
		return res;
	}
    
    
	// wandelt ein Byte-Paar gem angegebenem AudioFormat in eine Integer-Zahl um
    protected static int toInt (byte b1, byte b2, AudioFormat format)
	{
   		int upper_byte, lower_byte;
   		if (format.isBigEndian())
   		{
	    	upper_byte = b1;
	      	lower_byte = b2;
	   	}
	   	else
	   	{
	      	upper_byte = b2;
	      	lower_byte = b1;
	   	}
	   	int res = (upper_byte << 8) | (lower_byte & 0xff);
	   	if (format.getEncoding() == AudioFormat.Encoding.PCM_UNSIGNED)
	   		res -= 0x7fff;
	   	return res;
	} 	   	   	
	
/*	public static void main (String[] argv)
	{
		AudioData ad = new AudioData(new File("c:\\windows\\media\\chord.wav"));
		int left[]  = ad.getLeftChannel();
		int right[] = ad.getLeftChannel();
		System.out.println("Kanle: " + ad.getNumChannels());
		for (int i=0; i < left.length; i++)
			System.out.println(left[i] + "   " + right[i]);
	}*/
}
